package com.huiju.estateagents.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.entity.TaPerson;
import com.huiju.estateagents.entity.TaPointsRecords;
import com.huiju.estateagents.entity.TdPointsRules;
import com.huiju.estateagents.event.PointsEvent;
import com.huiju.estateagents.mapper.TaPersonMapper;
import com.huiju.estateagents.mapper.TaPointsRecordsMapper;
import com.huiju.estateagents.mapper.TdPointsRulesMapper;
import com.huiju.estateagents.service.ITaPointsRecordsService;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Map;

/**
 * <p>
 * 积分消费流水表  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-07-25
 */
@Service
public class TaPointsRecordsServiceImpl extends ServiceImpl<TaPointsRecordsMapper, TaPointsRecords> implements ITaPointsRecordsService {
	@Autowired
	TaPersonMapper taPersonMapper;
	
	@Autowired
	private TdPointsRulesMapper tdPointsRulesMapper;
	
	@Autowired
	private TaPointsRecordsMapper taPointsRecordsMapper;
	
	@Override
	public void sharePoints(String personId) {
		TaPerson person = taPersonMapper.selectById(personId);
		sendPoints(person,2,CommConstant.POINTS_SHARE);
	}
	
	@Override
	public void authPoints(String personId) {
		TaPerson person = taPersonMapper.selectById(personId);
		sendPoints(person,4,CommConstant.POINTS_AUTHORIZE);
	}
	
	private void sendPoints(TaPerson person,Integer rulesId,String shareType) {
		//发放积分
		QueryWrapper<TdPointsRules> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("rule_id",rulesId);
		queryWrapper.eq("status", CommConstant.STATUS_NORMAL);
		TdPointsRules tdPointsRules = tdPointsRulesMapper.selectOne(queryWrapper);
		if (null != tdPointsRules) {
			//插入积分消费流水表
			TaPointsRecords taPointsRecords = new TaPointsRecords();
			taPointsRecords.setPersonId(person.getPersonId());
			taPointsRecords.setPersonName(com.huiju.estateagents.common.StringUtils.ifNull(person.getName(),person.getNickname()));
			taPointsRecords.setPersonType(person.getPersonType());
			taPointsRecords.setPointsAmount(tdPointsRules.getPointsAmount());
			taPointsRecords.setChangeType(shareType);
			JSONObject jsonObject = new JSONObject();
			jsonObject.put("person_id",person.getPersonId());
			taPointsRecords.setChangeParams(jsonObject.toJSONString());
			taPointsRecords.setCreateDate(LocalDateTime.now());
			taPointsRecords.setStatus(CommConstant.STATUS_NORMAL);
			taPointsRecordsMapper.insert(taPointsRecords);
			//添加积分
			UpdateWrapper<TaPerson> taPersonwrapper = new UpdateWrapper<>();
			taPersonwrapper.eq("person_id", person.getPersonId());
			taPersonwrapper.setSql("points = IFNULL(points, 0) + " + String.valueOf(tdPointsRules.getPointsAmount()));
			taPersonMapper.update(new TaPerson(), taPersonwrapper);
		}
	}
}
