package com.huiju.estateagents.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.*;
import com.huiju.estateagents.event.PointsEvent;
import com.huiju.estateagents.mapper.*;
import com.huiju.estateagents.service.ITaShareService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户访问表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-05-10
 */
@Service
public class TaShareServiceImpl extends ServiceImpl<TaShareMapper, TaShare> implements ITaShareService {
    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    TaShareMapper taShareMapper;
    
    @Autowired
    private TdPointsRulesMapper tdPointsRulesMapper;
    
    @Autowired
    private TaPointsRecordsMapper taPointsRecordsMapper;

    @Autowired
    private TaBuildingMapper taBuildingMapper;

    @Autowired
    private TaBuildingDynamicMapper taBuildingDynamicMapper;
    
    @Resource
    private ApplicationContext applicationContext;

    @Override
    public ResponseBean saveData(String openid, String typeOf, String id) {
        List<TaPerson> persons = getPersonsByOpenId(openid);
        if (null == persons || persons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaPerson taPerson = persons.get(0);
        // 如果是分享活动, 则活动 + 1
        if (typeOf.equals(CommConstant.FAVOR_ACTIVITY)) {
            QueryWrapper<TaBuildingDynamic> taBuildingDynamicQuery= new QueryWrapper<>();
            taBuildingDynamicQuery.eq("dynamic_id",id);
            TaBuildingDynamic taBuildingDynamic = taBuildingDynamicMapper.selectOne(taBuildingDynamicQuery);
            taBuildingDynamicMapper.updateById( taBuildingDynamic.setShareNum(taBuildingDynamic.getShareNum()+1));
            return ResponseBean.success("成功");
        }

        QueryWrapper<TaShare> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("person_id", taPerson.getPersonId());
        queryWrapper.eq("be_share", id);
        queryWrapper.eq("tagert_type", typeOf);
        List<TaShare> uvs = taShareMapper.selectList(queryWrapper);
        if (null != uvs && uvs.size() > 0) {
            return ResponseBean.success("");
        }

        TaShare taShare = new TaShare();
        taShare.setBeShare(id);
        taShare.setCreateDate(LocalDateTime.now());
        taShare.setNameOrnick(taPerson.getNickname());
        taShare.setPersonId(taPerson.getPersonId());
        taShare.setPhotoOravatar(taPerson.getAvatarurl());
        taShare.setTagertType(typeOf);

        int row = taShareMapper.insert(taShare);
        if (row < 1) {
            return ResponseBean.error("分享增加失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 如果是分享置业, 则人员分享 + 1
        if (typeOf == CommConstant.FAVOR_CONSULTANT) {
            taPersonMapper.setFieldIncrement(id,"share_num", 1);
        }

        if(typeOf.equals(CommConstant.FAVOR_PROJECT)) {
            taBuildingMapper.setFieldNum(id, "share_num", 1);
        }

        //发放分享积分
        if (typeOf.equals(CommConstant.FAVOR_PROJECT)){
            applicationContext.publishEvent(new PointsEvent(taPerson.getPersonId(),CommConstant.POINTS_SHARE));
        }
        return ResponseBean.success("");
    }

    @Override
    public ResponseBean getListOf(String id, String typeOf, Integer pageNumber, Integer pageSize) {
        IPage<TaShare> page = new Page(pageNumber, pageSize);
        QueryWrapper<TaShare> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("be_share", id);
        queryWrapper.eq("tagert_type", typeOf);
        queryWrapper.orderByDesc("create_date");

        IPage<TaShare> result = taShareMapper.selectPage(page, queryWrapper);
        return ResponseBean.success(result);
    }


    List<TaPerson> getPersonsByOpenId(String openid) {
        return taPersonMapper.getPersonByOpenId(openid);
    }
}
